import pkgutil
import os
import sys

def find_package(import_name): 
    #copy from flask
    """Finds a package and returns the prefix (or None if the package is
    not installed) as well as the folder that contains the package or
    module as a tuple.  The package path returned is the module that would
    have to be added to the pythonpath in order to make it possible to
    import the module.  The prefix is the path below which a UNIX like
    folder structure exists (lib, share etc.).
    """
    root_mod_name = import_name.split('.')[0]
    loader = pkgutil.get_loader(root_mod_name)
    if loader is None or import_name == '__main__':
        # import name is not found, or interactive/main module
        package_path = os.getcwd()
    else:
        # For .egg, zipimporter does not have get_filename until Python 2.7.
        if hasattr(loader, 'get_filename'):
            filename = loader.get_filename(root_mod_name)
        elif hasattr(loader, 'archive'):
            # zipimporter's loader.archive points to the .egg or .zip
            # archive filename is dropped in call to dirname below.
            filename = loader.archive
        else:
        # At least one loader is missing both get_filename and archive:
        # Google App Engine's HardenedModulesHook
        #
        # Fall back to imports.
            __import__(import_name)
            filename = sys.modules[import_name].__file__
        package_path = os.path.abspath(os.path.dirname(filename))
        # package_path ends with __init__.py for a package
        if loader.is_package(root_mod_name):
            package_path = os.path.dirname(package_path)

    site_parent, site_folder = os.path.split(package_path)
    py_prefix = os.path.abspath(sys.prefix)
    if package_path.startswith(py_prefix):
        return py_prefix, package_path
    elif site_folder.lower() == 'site-packages':
        parent, folder = os.path.split(site_parent)
        # Windows like installations
        if folder.lower() == 'lib':
            base_dir = parent
        # UNIX like installations
        elif os.path.basename(parent).lower() == 'lib':
            base_dir = os.path.dirname(parent)
        else:
            base_dir = site_parent
            return base_dir, package_path
    return None, package_path


def getDefaultPath():
    prefix, package_path = find_package(__name__)
    if prefix is None:
        return package_path
    root_mod_name = __name__.split('.')[0]
    return os.path.join(prefix, 'var', root_mod_name)



import os
from .renderer import render_content
from .renderer import render_with_template
from .renderer import render_toc
from .renderer import render_index
from .renderer import fix_file_link
from .renderer import init_env
from .util import getDefaultPath
from datetime import datetime
import settings 
import requests
import re
import sys
import shutil
import codecs
import json

def transform(paths = None, cache_path = None, system_css = False, css = False, abscss = False, gfm = False, username = None, password = None, needtoc = True, toc_depth = None, toc_file = None, book = '', offline = False, encoding = 'utf-8', refresh = False, file_reg = None, template_path = None):

    #first, initial enviroment for jinjia2
    init_env(template_path)

    if len(paths) == 0:
        paths = ['.']

    #Get style file
    styles, style_paths = get_style(cache_path, system_css, refresh)
    #compile file reg exp
    file_reg = '\.(md|markdown)$' if file_reg is None else file_reg
    file_reg = re.compile(file_reg, re.I|re.U)

    render_flist = []

    for path in paths:
	if not os.path.exists(path):
	    print "File not found: " + path
	    continue
	#TODO:
	#    add recursive support
	if os.path.isdir(path):
	    path_files = os.listdir(path)
            path_files = [os.path.join(path, f) for f in path_files if file_reg.search(f)]
            render_flist.extend([os.path.abspath(f) for f in path_files if os.path.isfile(f)])
        elif os.path.isfile(path):
                render_flist.append(os.path.abspath(path))
        else:
                raise ValueError('Not supported file: ' + path)
    
    #print "in render_flist"
    #for f in render_flist:
      #print f

    print len(render_flist), " files in render list..." 

    contents = []
    tocs = []

    #get all file rendered using github api or offline renderer
    #Also, get toc
    for i, f in enumerate(render_flist):
        print i+1, "/", len(render_flist), ": ",
        content, toc, extradata = render_content(f, gfm, username, password, needtoc, offline, encoding)
        htmlname = __get_htmlfilename(f)
        contents.append([htmlname, content, f])
        if needtoc:
            tocs.extend(__process_toc(toc, htmlname))

        print "done."

    #fix relative links: 01.md => 01.html
    for i, info in enumerate(contents): 
        contents[i][1] = fix_file_link(info[1], info[2], contents, file_reg)

    if needtoc:
        if toc_file:
            print 'toc_file:', toc_file
            toc_file = os.path.abspath(toc_file)
            print 'Generating custom toc'
            rtoc, toc, extradata = render_content(toc_file, gfm, username, password, False, offline, encoding)
            print 'done.'
            rtoc = fix_file_link(rtoc, toc_file, contents, file_reg)
        else:
            rtoc = render_toc(tocs, toc_depth)

        ##after render toc, we render index
        bookinfo = __get_book_conf(book)
        print 'Generating index.html'
        if bookinfo:
            book_index = render_index(bookinfo['title'], bookinfo['coverimage'], bookinfo['description'], rtoc, True if toc_file else False)
        else:
            book_index = render_index('', '', '', rtoc, True if toc_file else False)
        with open('index.html', 'w') as f:
            f.write(book_index.encode('utf-8'))
    else:
        rtoc = None

    #After get all file rendered, render them with template & save into files
    for i in range(len(contents)):
        p = contents[i - 1][0] if i > 0 else None 
        n = contents[i + 1][0] if i + 1 != len(contents) else None

        rendered = render_with_template('', contents[i][1], rtoc, p, n, css, abscss, needtoc, styles, style_paths)
        with open(contents[i][0], 'w') as f:
            f.write(rendered.encode('utf-8'))

    print 'All finished'
    if extradata:
        print "Github API rate remains: ", extradata['x-ratelimit-remaining'], "/", extradata['x-ratelimit-limit'], "."
        print "Reset at: ", datetime.fromtimestamp(float(extradata['x-ratelimit-reset']))



def __get_book_conf(book):
    if book:
        book = os.path.expanduser(book)
        if not os.path.exists(book):
            raise RuntimeError(book+' not exists')
        with open(book, 'r') as f:
            bookinfo = json.load(f)
        return bookinfo
    return None 


def __get_htmlfilename(path):
    basename = os.path.basename(path)
    filename = re.split('\.(markdown|md)', basename, re.U)[0]
    return unicode(filename + '.html', 'utf-8')


def __process_toc(toc, htmlname):
    ##process toc, add htmlname into link
    for header in toc:
        header[2] = htmlname + header[2]
    return toc



def _get_cached_style_files(cache_path):
    """Gets the URLs of the cached styles."""
    cached_styles = os.listdir(cache_path)
    return [os.path.join(cache_path, style) for style in cached_styles]


def _cache_style(urls, cache_path):
    """Fetches the given URLs and caches their contents in the given directory."""
    for url in urls:
        basename = url.rsplit('/', 1)[-1]
        print '\tDownload css file: ', basename, '...',
        file.flush(sys.stdout)
        filename = os.path.join(cache_path, basename)
        contents = requests.get(url).text
        with open(filename, 'w') as f:
            f.write(contents.encode('utf-8'))
        print 'done'
        file.flush(sys.stdout)


def _get_style_urls(cache_path):
    '''Get css urls from settings.STYLE_URLS_SOURCE
        if css files are already cached, return []
    '''
    try:
        cached = _get_cached_style_files(cache_path)
        if not cached:
            # Find css url
            print "Github css files are not cached. Download First"
            print "Fetching css url from ", settings.STYLE_URLS_SOURCE,
            file.flush(sys.stdout)
            r = requests.get(settings.STYLE_URLS_SOURCE)
            if not 200 <= r.status_code < 300:
                print ' * Warning: retrieving styles gave status code', r.status_code
                raise RuntimeError('Get css file failed')
            urls = re.findall(settings.STYLE_URLS_RE, r.text)
            print "......done"
            return urls

    except Exception as ex:
        print '* Error: Retrive style error:', str(ex)
    return []


def _get_style_contents(urls, cache_path):
    '''fetching css file content, cache if not exists
       return contents, file_paths
    '''
    styles = []
    file_paths = []
    cached_styles = os.listdir(cache_path)
    for url in urls:
        basename = url.rsplit('/', 1)[-1]
        if basename not in cached_styles:
            _cache_style([url], cache_path)

    cached_styles = os.listdir(cache_path)

    for style in cached_styles:
        basename = style.rsplit('/', 1)[-1]
        css_path = os.path.join(cache_path, basename)
        with codecs.open(css_path, mode='r', encoding='utf-8') as f:
            styles.append(f.read())
        file_paths.append(css_path)
    return styles, file_paths


def get_style(cache_path, system_css, refresh):
    '''Get github's css, render to html file later
        return style content
    '''
    if cache_path is None:
        cache_path = os.path.curdir
    if system_css:
        cache_path = getDefaultPath()

    cache_path = os.path.expanduser(cache_path)

    cache_path = os.path.join(cache_path, 'style_cache')
    
    #make a clean cache_path
    if refresh:
        shutil.rmtree(cache_path)

    if not os.path.exists(cache_path):
        os.makedirs(cache_path, 0755)

    style_urls = settings.STYLE_URLS[:]
    style_urls.extend(_get_style_urls(cache_path))
    styles, style_paths = _get_style_contents(style_urls, cache_path)
    return styles, style_paths




import requests
import json


def github_render_content(text, gfm=False, context=None, username=None, password=None):
    """Renders the specified markup using the GitHub API."""
    if gfm:
        url = 'https://api.github.com/markdown'
        data = {'text': text, 'mode': 'gfm', 'context': context}
        if context:
            data['context'] = context
        data = json.dumps(data)
    else:
        url = 'https://api.github.com/markdown/raw'
        data = text
    headers = {'content-type': 'text/plain'}
    auth = (username, password) if username else None

    r = requests.post(url, headers=headers, data=data, auth=auth)

    extradata = None
    message = None
    # Relay HTTP errors
    if r.status_code != 200:
        try:
            message = r.json()['message']
        except:
            message = r.text
    extradata = {'x-ratelimit-remaining': r.headers['x-ratelimit-remaining'],
                 'x-ratelimit-limit': r.headers['x-ratelimit-limit'],
                 'x-ratelimit-reset': r.headers['x-ratelimit-reset']
            }

    return r.text, message, extradata



from urllib import quote_plus
import markdown
import codecs
from bs4 import BeautifulSoup

def get_toc(filename, encoding):
	#get toc
	input_file = codecs.open(filename, mode='r', encoding=encoding)
	text = input_file.read()
	input_file.close()
	md = markdown.Markdown(extensions=['toc', 'fenced_code'], extension_configs={'toc': [('slugify', github_link_renderer)]})
	md.convert(text)

	#convert to toc list
	soup = BeautifulSoup(md.toc)
	tocs = _toc(soup.select('div > ul > li'))

	return tocs 


def _toc(toc_list, level = 1):
	menus = []
	for li in toc_list:
		menu = [ ['h%d' % level, li.a.get_text(), li.a.attrs['href']] ]
		#menu = [ [li.a.get_text(), li.a.attrs['href']] ]
		if li.ul is not None:
			menu.extend(_toc(li.ul.find_all('li', recursive=False), level+1))
		menus.extend(menu)
	return menus


def github_link_renderer(link, md):
	'''
		get github-like heading href.
	'''
	link = link.lower().replace(' ', '-').replace('+', '')
	return quote_plus(link.encode('utf-8'))


def get_github_toc(content):
	soup = BeautifulSoup(content)
	toclinks = soup.select('a.anchor span.octicon.octicon-link')
	tocs = []
	for link in toclinks:
		a = link.parent
		h = a.parent
		tocs.append([h.name, h.get_text(strip=True), a.attrs['href']])

	return tocs





from .github_renderer import github_render_content
from .toc import get_toc
from .toc import get_github_toc
from jinja2 import Environment, PackageLoader, FileSystemLoader
from bs4 import BeautifulSoup
from urllib import url2pathname
import os.path
import sys
import re

##for jinjia2
##Get template to render
env = None
content_template = None
toc_template = None
index_template = None

def init_env(template_path):
    global env, content_template, toc_template, index_template
    #global content_template
    #global toc_template
    #global index_template
    if template_path:
        env = Environment(loader=FileSystemLoader(template_path), 
                extensions=['jinja2.ext.do', 'jinja2.ext.loopcontrols'])
    else:
        env = Environment(loader=PackageLoader('mdtogh', 'templates'),
                extensions=['jinja2.ext.do', 'jinja2.ext.loopcontrols'])
    content_template = env.get_template('content.html')
    toc_template = env.get_template('toc.html')
    index_template = env.get_template('index.html')


def render_content(filename, gfm, username, password, toc, offline, encoding):
    '''render one file
        return: content, toc	
    '''
    print 'Rendering: ', filename,
    file.flush(sys.stdout)

    if offline:
        #offline_renderer, using get_toc to get toc
        if encoding is None:
            encoding = 'utf-8'
        content = ''
        gentoc = get_toc(filename, encoding)
        extradata = None
        pass
    else:
        ##using github renderer
        with open(filename) as f:
            content, message, extradata = github_render_content(f.read(), gfm, None, username, password)
            if message != None:
                raise RuntimeError('render file error: ' + message)

        gentoc = None
        if toc:
            gentoc = get_github_toc(content)

    return content, gentoc, extradata


def render_with_template(title, content, toc, prevfile, nextfile, css, abscss, needtoc, styles, style_paths):
    '''
        render file using template
    '''
    #if using css, then clear styles
    #otherwise, clear style_paths
    if css:
        styles[:] = []
        if not abscss:
            style_paths = [os.path.relpath(path) for path in style_paths]
    else:
        style_paths[:] = []

    return content_template.render(content=content, filetitle=title,
        style_paths=style_paths, styles=styles, toc = toc, needtoc = needtoc, 
        prevfile = prevfile, nextfile = nextfile)


def render_toc(tocs, toc_depth):
    if toc_depth is None:
        toc_depth = '2'

    if not toc_depth.isdigit():
        raise ValueError('--toc-depth must be digit')

    return toc_template.render(tocs = tocs, toc_depth = toc_depth)


def render_index(title, cover, description, toc, custom_toc):
    return index_template.render(booktitle = title, coverimage = cover,
            description = description, toc = toc, custom_toc = custom_toc)


def fix_file_link(fcontent, f, contents, file_reg):
    #fix link in file
    #fcontent: content of file to be fixed
    #f: name of file to be fixed
    #contents: files after render
    hrefreg = re.compile('^(http://|https://)')

    soup = BeautifulSoup(fcontent)
    hrefs = soup.find_all('a', href=file_reg)
    for href in hrefs:
        #fix only relative path
        if hrefreg.search(href['href']) is None:
            newpath = os.path.normpath(os.path.join(os.path.dirname(f), url2pathname(href['href'])))
            if not os.path.exists(newpath):
                print 'warning: link in ', os.path.basename(f), ': ', href['href'], ' not exists..'
            else:
                htmlname = [info[0] for info in contents if info[2] == newpath]
                if htmlname:
                    href['href'] = htmlname[0]
                else:
                    print 'warning: link in ', os.path.basename(info[2]), ': ', href['href'], ' not in render list'

    return soup.prettify() 




"""\
mdtogh.command
-------------------

Implements the command-line interface for md to github.


Usage:
  mdtogh [options] [<path>] ...
  mdtogh -h | --help
  mdtogh --version

Where:
  <path> is a file or a directory to render, [default: '.']

Options:
  --templates=<path>    path of templates, it should contains all three files: content.html, toc.html, index.html
  --cache_path=<path>  path to store style file cache, default to current directory
  --system_css         using system wide css.
  --css                when NOT set, css contents are generate into html
  --abscss             link css with absolute path, use only with --css is set 
  --gfm                Use GitHub-Flavored Markdown, e.g. comments or issues
  --context=<repo>     The repository context, only taken into account with --gfm
  --user=<username>    GitHub username for API authentication
  --pass=<password>    GitHub password for API authentication
  --toc                Generate table of contents
  --toc_depth=<n>      Max toc depth, default to 2
  --toc_file=<file>    You can specific a file to be toc(Just incase you have one)
  --book=<book.json>   Generate toc with book info, only used when --toc is set
  --offline            Use offline renderer
  --encoding=<utf-8>   encode for file, use only when --offline is set
  --refresh            clear cached styles & refetch them
  --file_reg=<reg_exp> when path is a directory, using reg_exp to get file, this reg_exp must obey python's rules
		       if not set, mdtogh will get all files end with .md or .markdown, Notice: this is case-insensitive.

Notice:
	Due to limitation by github, the rate of anonymous access to github api is limit to 60 in an hour.
	If you need to generate more than that, please set --user & --pass.
	Also, --user & --pass are sended via https.
"""

import sys
from docopt import docopt
from .transform import transform
from . import __version__
import jinja2

usage = '\n\n\n'.join(__doc__.split('\n\n\n')[1:])

#TODO:
#add custom title support
#add recursive support
#add custom template support

def main(argv=None):
    """Entry point of this application"""
    if argv is None:
        argv = sys.argv[1:]
        version = 'mdtogh ' + __version__

        args = docopt(usage, argv=argv, version=version)
        #json.dump(args, sys.stdout)

    try:
        transform(args['<path>'], args['--cache_path'], args['--system_css'], args['--css'], args['--abscss'], 
                args['--gfm'], args['--user'],args['--pass'], 
                args['--toc'], args['--toc_depth'], args['--toc_file'], args['--book'], 
                args['--offline'], args['--encoding'], args['--refresh'], args['--file_reg'], args['--templates'])
        return 0
    except ValueError as e:
      print "Error: ", e
      return 1
    except RuntimeError as e:
       print "Error: ", e
       return 1
    except jinja2.TemplateNotFound as e:
        print "Template not found ", e, "...Invalid templates path given or template miss."
    except jinja2.TemplateSyntaxError as e:
        print "Syntax Error in template ", e.args[3] if len(e.args) > 3 else '', " line "+e.args[1] if len(e.args) > 1 else '', ": ", e.args[0]
    except jinja2.TemplateError as e:
        print "jinja2 Error: ", e
    except Exception as e:
        print "Error: ", e.args
        return 1


if __name__ == '__main__':
    main()



import markdown
import codecs

def offline_renderer(filename, encoding):
	#get toc
    with codecs.open(filename, mode='r', encoding=encoding) as f:
        text = f.read()
	return markdown.markdown(text, 
            extensions=['fenced_code'])




import os
import sys
from setuptools import setup, find_packages


if sys.argv[-1] == 'publish':
    sys.exit(os.system('python setup.py sdist upload'))



def read(fname):
   with open(os.path.join(os.path.dirname(__file__), fname)) as f:
       return f.read()


setup(
    name='mdtogh',
    version='0.0.7',
    packages=find_packages(),
    #install requirements
    install_requires=read('requirements.txt'),

    package_data={'mdtogh': ['templates/*']},
    include_package_data=True,

    entry_points={'console_scripts': ['mdtogh = mdtogh.command:main']},

    zip_safe=False,

    author='Summer Ruan',
    author_email='marchtea213@gmail.com',
    description='Transform markdown files into html with styles of github',
    long_description=read('descriptions.rst'),
    url='http://github.com/marchtea/mdtogh',
    platforms='any',
)



STYLE_URLS = []
STYLE_URLS_SOURCE = "https://github.com/marchtea/md_to_github_html"
STYLE_URLS_RE = '<link.+href=[\'"]?([^\'" >]+)[\'"]?.+media=[\'"]?(?:screen|all)[\'"]?.+rel=[\'"]?stylesheet[\'"]?.+/>'
